@@ -1,5 +1,5 @@
 //
-// Copyright (C) 2010-2017 Codership Oy <info@codership.com>
+// Copyright (C) 2010-2020 Codership Oy <info@codership.com>
 //
 
 #include "trx_handle.hpp"
@@ -50,7 +50,7 @@ void check_states_graph(
 
     for (int i(0); i < TrxHandle::num_states_; ++i)
     {
-        fail_unless(visited[i] == visits[i],
+        ck_assert_msg(visited[i] == visits[i],
                     "i = %i visited = %i visits = %i",
                     i, visited[i], visits[i]);
     }
@@ -73,7 +73,7 @@ START_TEST(test_states_master)
                            TrxHandleMasterDeleter());
     galera::TrxHandleLock lock(*trx);
 
-    fail_unless(trx->state() == TrxHandle::S_EXECUTING);
+    ck_assert(trx->state() == TrxHandle::S_EXECUTING);
 
     // Matrix representing directed graph of TrxHandleMaster transitions,
     // see galera/src/trx_handle.cpp
@@ -138,7 +138,7 @@ START_TEST(test_states_slave)
 
     TrxHandleSlavePtr ts(TrxHandleSlave::New(false, sp),
                          TrxHandleSlaveDeleter());
-    fail_unless(ts->state() == TrxHandle::S_REPLICATING);
+    ck_assert(ts->state() == TrxHandle::S_REPLICATING);
 
     std::vector<int> visits(TrxHandle::num_states_);
     std::fill(visits.begin(), visits.end(), 0);
@@ -169,15 +169,15 @@ START_TEST(test_serialization)
 
         std::vector<gu::byte_t> buf;
         trx->serialize(0, buf);
-        fail_unless(buf.size() > 0);
+        ck_assert(buf.size() > 0);
 
         TrxHandleSlavePtr txs1(TrxHandleSlave::New(false, sp),
                                TrxHandleSlaveDeleter());
         gcs_action const act =
             { 1, 2, buf.data(), int(buf.size()), GCS_ACT_WRITESET};
-        fail_unless(txs1->unserialize<true>(act) > 0);
-        fail_if(txs1->global_seqno() != act.seqno_g);
-        fail_if(txs1->local_seqno()  != act.seqno_l);
+        ck_assert(txs1->unserialize<true>(act) > 0);
+        ck_assert(txs1->global_seqno() == act.seqno_g);
+        ck_assert(txs1->local_seqno()  == act.seqno_l);
     }
 }
 END_TEST
@@ -193,11 +193,11 @@ apply_cb(
     )
 {
     std::vector<char>* const res(static_cast<std::vector<char>* >(ctx));
-    fail_if(NULL == res);
+    ck_assert(NULL != res);
 
     const char* const c(static_cast<const char*>(data->ptr));
-    fail_if(NULL == c);
-    fail_if(1 != data->len);
+    ck_assert(NULL != c);
+    ck_assert(1 == data->len);
 
     res->push_back(*c);
 
@@ -223,9 +223,9 @@ START_TEST(test_streaming)
     src[0] = 'a'; src[1] = 'b'; src[2] = 'c';
 
     std::vector<char> res;          // apply_cb should reproduce src in res
-    fail_if(src == res);
+    ck_assert(src != res);
 
-    fail_unless(trx->flags() & TrxHandle::F_BEGIN);
+    ck_assert(trx->flags() & TrxHandle::F_BEGIN);
 
     {
         // 0. first fragment A
@@ -235,16 +235,16 @@ START_TEST(test_streaming)
         std::vector<gu::byte_t> buf;
         trx->serialize(0, buf);
 
-        fail_unless(buf.size() > 0);
+        ck_assert(buf.size() > 0);
         trx->release_write_set_out();
 
         TrxHandleSlavePtr ts(TrxHandleSlave::New(false, sp),
                              TrxHandleSlaveDeleter());
         gcs_action const act =
             { 1, 2, buf.data(), int(buf.size()), GCS_ACT_WRITESET};
-        fail_unless(ts->unserialize<true>(act) > 0);
-        fail_unless(ts->flags() & TrxHandle::F_BEGIN);
-        fail_if(ts->flags() & TrxHandle::F_COMMIT);
+        ck_assert(ts->unserialize<true>(act) > 0);
+        ck_assert(ts->flags() & TrxHandle::F_BEGIN);
+        ck_assert(!(ts->flags() & TrxHandle::F_COMMIT));
         trx->add_replicated(ts);
 
         wsrep_bool_t exit_loop;
@@ -258,16 +258,16 @@ START_TEST(test_streaming)
 
         std::vector<gu::byte_t> buf;
         trx->serialize(0, buf);
-        fail_unless(buf.size() > 0);
+        ck_assert(buf.size() > 0);
         trx->release_write_set_out();
 
         TrxHandleSlavePtr ts(TrxHandleSlave::New(false, sp),
                              TrxHandleSlaveDeleter());
         gcs_action const act =
             { 2, 3, buf.data(), int(buf.size()), GCS_ACT_WRITESET};
-        fail_unless(ts->unserialize<true>(act) > 0);
-        fail_if(ts->flags() & TrxHandle::F_BEGIN);
-        fail_if(ts->flags() & TrxHandle::F_COMMIT);
+        ck_assert(ts->unserialize<true>(act) > 0);
+        ck_assert(!(ts->flags() & TrxHandle::F_BEGIN));
+        ck_assert(!(ts->flags() & TrxHandle::F_COMMIT));
         trx->add_replicated(ts);
 
         wsrep_bool_t exit_loop;
@@ -282,23 +282,23 @@ START_TEST(test_streaming)
 
         std::vector<gu::byte_t> buf;
         trx->serialize(0, buf);
-        fail_unless(buf.size() > 0);
+        ck_assert(buf.size() > 0);
         trx->release_write_set_out();
 
         TrxHandleSlavePtr ts(TrxHandleSlave::New(false, sp),
                              TrxHandleSlaveDeleter());
         gcs_action const act =
             { 3, 4, buf.data(), int(buf.size()), GCS_ACT_WRITESET};
-        fail_unless(ts->unserialize<true>(act) > 0);
-        fail_if(ts->flags() & TrxHandle::F_BEGIN);
-        fail_unless(ts->flags() & TrxHandle::F_COMMIT);
+        ck_assert(ts->unserialize<true>(act) > 0);
+        ck_assert(!(ts->flags() & TrxHandle::F_BEGIN));
+        ck_assert(ts->flags() & TrxHandle::F_COMMIT);
         trx->add_replicated(ts);
 
         wsrep_bool_t exit_loop;
         ts->apply(&res, apply_cb, wsrep_trx_meta_t(), exit_loop);
     }
 
-    fail_if(res != src);
+    ck_assert(res == src);
 }
 END_TEST
 
