@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2018 Codership Oy <info@codership.com>
+ * Copyright (C) 2010-2020 Codership Oy <info@codership.com>
  */
 
 #include "write_set.hpp"
@@ -11,6 +11,8 @@
 #include "gcs_action_source.hpp"
 #include "galera_service_thd.hpp"
 
+#include "gu_inttypes.hpp"
+
 #include <cstdlib>
 #include <check.h>
 
@@ -104,17 +106,17 @@ START_TEST(test_key1)
     expected_size += gu::uleb128_size(expected_size);
 #endif
 
-    fail_unless(key.serial_size() == expected_size, "%ld <-> %ld",
-                key.serial_size(), expected_size);
+    ck_assert_msg(key.serial_size() == expected_size, "%ld <-> %ld",
+                  key.serial_size(), expected_size);
 
     KeyPartSequence kp(key.key_parts<KeyPartSequence>());
-    fail_unless(kp.size() == 4);
+    ck_assert(kp.size() == 4);
 
     gu::Buffer buf(key.serial_size());
     key.serialize(&buf[0], buf.size(), 0);
     KeyOS key2(1);
     key2.unserialize(&buf[0], buf.size(), 0);
-    fail_unless(key2 == key);
+    ck_assert(key2 == key);
 }
 END_TEST
 
@@ -156,17 +158,17 @@ START_TEST(test_key2)
     expected_size += gu::uleb128_size(expected_size);
 #endif
 
-    fail_unless(key.serial_size() == expected_size, "%ld <-> %ld",
-                key.serial_size(), expected_size);
+    ck_assert_msg(key.serial_size() == expected_size, "%ld <-> %ld",
+                  key.serial_size(), expected_size);
 
     KeyPartSequence kp(key.key_parts<KeyPartSequence>());
-    fail_unless(kp.size() == 4);
+    ck_assert(kp.size() == 4);
 
     gu::Buffer buf(key.serial_size());
     key.serialize(&buf[0], buf.size(), 0);
     KeyOS key2(2);
     key2.unserialize(&buf[0], buf.size(), 0);
-    fail_unless(key2 == key);
+    ck_assert(key2 == key);
 }
 END_TEST
 
@@ -214,14 +216,14 @@ START_TEST(test_write_set1)
         + 1 + 1 + 6 + 1 + 4 // key2
 #endif
         + 4 + 6; // rbr
-    fail_unless(buf.size() == expected_size, "%zd <-> %zd <-> %zd",
-                buf.size(), expected_size, ws.serial_size());
+    ck_assert_msg(buf.size() == expected_size, "%zd <-> %zd <-> %zd",
+                  buf.size(), expected_size, ws.serial_size());
 
 
     WriteSet ws2(0);
 
     size_t ret = ws2.unserialize(&buf[0], buf.size(), 0);
-    fail_unless(ret == expected_size);
+    ck_assert(ret == expected_size);
 
     WriteSet::KeySequence rks;
     ws.get_keys(rks);
@@ -229,9 +231,9 @@ START_TEST(test_write_set1)
     WriteSet::KeySequence rks2;
     ws.get_keys(rks2);
 
-    fail_unless(rks2 == rks);
+    ck_assert(rks2 == rks);
 
-    fail_unless(ws2.get_data() == ws.get_data());
+    ck_assert(ws2.get_data() == ws.get_data());
 
 }
 END_TEST
@@ -280,14 +282,14 @@ START_TEST(test_write_set2)
         + 1 + 1 + 6 + 1 + 4 // key2
 #endif
         + 4 + 6; // rbr
-    fail_unless(buf.size() == expected_size, "%zd <-> %zd <-> %zd",
-                buf.size(), expected_size, ws.serial_size());
+    ck_assert_msg(buf.size() == expected_size, "%zd <-> %zd <-> %zd",
+                  buf.size(), expected_size, ws.serial_size());
 
 
     WriteSet ws2(2);
 
     size_t ret = ws2.unserialize(&buf[0], buf.size(), 0);
-    fail_unless(ret == expected_size);
+    ck_assert(ret == expected_size);
 
     WriteSet::KeySequence rks;
     ws.get_keys(rks);
@@ -295,9 +297,9 @@ START_TEST(test_write_set2)
     WriteSet::KeySequence rks2;
     ws2.get_keys(rks2);
 
-    fail_unless(rks2 == rks);
+    ck_assert(rks2 == rks);
 
-    fail_unless(ws2.get_data() == ws.get_data());
+    ck_assert(ws2.get_data() == ws.get_data());
 
 }
 END_TEST
@@ -317,7 +319,7 @@ START_TEST(test_mapped_buffer)
     mb.resize(1 << 8);
     for (size_t i = 0; i < 16; ++i)
     {
-        fail_unless(mb[i] == static_cast<gu::byte_t>(i));
+        ck_assert(mb[i] == static_cast<gu::byte_t>(i));
     }
 
     for (size_t i = 16; i < (1 << 8); ++i)
@@ -329,7 +331,7 @@ START_TEST(test_mapped_buffer)
 
     for (size_t i = 0; i < (1 << 8); ++i)
     {
-        fail_unless(mb[i] == static_cast<gu::byte_t>(i));
+        ck_assert(mb[i] == static_cast<gu::byte_t>(i));
     }
 
     for (size_t i = 0; i < (1 << 20); ++i)
@@ -437,12 +439,13 @@ START_TEST(test_cert_hierarchical_v1)
 
         trx->set_received(0, wsi[i].local_seqno, wsi[i].global_seqno);
         Certification::TestResult result(cert.append_trx(trx));
-        fail_unless(result == wsi[i].result, "wsi: %zu, g: %lld r: %d er: %d",
-                    i, trx->global_seqno(), result, wsi[i].result);
-        fail_unless(trx->depends_seqno() == wsi[i].expected_depends_seqno,
-                    "wsi: %zu g: %lld ld: %lld eld: %lld",
-                    i, trx->global_seqno(), trx->depends_seqno(),
-                    wsi[i].expected_depends_seqno);
+        ck_assert_msg(result == wsi[i].result,
+                      "wsi: %zu, g: %" PRId64 " r: %d er: %d",
+                      i, trx->global_seqno(), result, wsi[i].result);
+        ck_assert_msg(trx->depends_seqno() == wsi[i].expected_depends_seqno,
+                      "wsi: %zu g: %" PRId64 " ld: %" PRId64 " eld: %" PRId64,
+                      i, trx->global_seqno(), trx->depends_seqno(),
+                      wsi[i].expected_depends_seqno);
         cert.set_trx_committed(trx);
         trx->unref();
     }
@@ -561,19 +564,23 @@ START_TEST(test_cert_hierarchical_v2)
 
         trx->set_received(0, wsi[i].local_seqno, wsi[i].global_seqno);
         Certification::TestResult result(cert.append_trx(trx));
-        fail_unless(result == wsi[i].result, "g: %lld res: %d exp: %d",
-                    trx->global_seqno(), result, wsi[i].result);
-        fail_unless(trx->depends_seqno() == wsi[i].expected_depends_seqno,
-                    "wsi: %zu g: %lld ld: %lld eld: %lld",
-                    i, trx->global_seqno(), trx->depends_seqno(),
-                    wsi[i].expected_depends_seqno);
+        ck_assert_msg(result == wsi[i].result, "g: %" PRId64 " res: %d exp: %d",
+                      trx->global_seqno(), result, wsi[i].result);
+        ck_assert_msg(trx->depends_seqno() == wsi[i].expected_depends_seqno,
+                      "wsi: %zu g: %" PRId64 " ld: %" PRId64 " eld: %" PRId64,
+                      i, trx->global_seqno(), trx->depends_seqno(),
+                      wsi[i].expected_depends_seqno);
         cert.set_trx_committed(trx);
         trx->unref();
     }
 }
 END_TEST
 
-
+// This test leaks memory and it is for trx protocol version 2
+// which is pre 25.3.5. Disabling this test for now with ASAN
+// build. The test should be either removed or fixed to work
+// with more recent protocol versions.
+#ifndef GALERA_WITH_ASAN
 START_TEST(test_trac_726)
 {
     log_info << "test_trac_726";
@@ -609,7 +616,7 @@ START_TEST(test_trac_726)
 
         trx->set_received(0, 1, 1);
         Certification::TestResult result(cert.append_trx(trx));
-        fail_unless(result == Certification::TEST_OK);
+        ck_assert(result == Certification::TEST_OK);
         cert.set_trx_committed(trx);
         trx->unref();
     }
@@ -635,13 +642,13 @@ START_TEST(test_trac_726)
 
         trx->set_received(0, 2, 2);
         Certification::TestResult result(cert.append_trx(trx));
-        fail_unless(result == Certification::TEST_FAILED);
+        ck_assert(result == Certification::TEST_FAILED);
         cert.set_trx_committed(trx);
         trx->unref();
     }
 }
 END_TEST
-
+#endif // GALERA_WITH_ASAN
 
 Suite* write_set_suite()
 {
@@ -675,13 +682,15 @@ Suite* write_set_suite()
 
     tc = tcase_create("test_cert_hierarchical_v2");
     tcase_add_test(tc, test_cert_hierarchical_v2);
-    tcase_set_timeout(tc, 20);
+    tcase_set_timeout(tc, 120);
     suite_add_tcase(s, tc);
 
+#ifndef GALERA_WITH_ASAN
     tc = tcase_create("test_trac_726");
     tcase_add_test(tc, test_trac_726);
     tcase_set_timeout(tc, 20);
     suite_add_tcase(s, tc);
+#endif
 
     return s;
 }
