@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2017 Codership Oy <info@codership.com>
+ * Copyright (C) 2010-2024 Codership Oy <info@codership.com>
  */
 
 /*! @file memory operations interface */
@@ -7,6 +7,7 @@
 #ifndef _gcache_memops_hpp_
 #define _gcache_memops_hpp_
 
+#include "gcache_seqno.hpp"
 #include <gu_arch.h>
 #include <gu_macros.h>
 #include <gu_limits.h> // GU_MIN_ALIGNMENT
@@ -49,6 +50,12 @@ namespace gcache
         virtual void
         reset   ()                        = 0;
 
+        virtual void
+        seqno_lock(seqno_t seqno_g)       = 0;
+
+        virtual void
+        seqno_unlock()                    = 0;
+
         /* GCache 3.x is not supposed to be portable between platforms */
         static size_type const ALIGNMENT  = GU_MIN_ALIGNMENT;
 
