@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2018 Codership Oy <info@codership.com>
+ * Copyright (C) 2010-2024 Codership Oy <info@codership.com>
  */
 
 /*! @file page store class */
@@ -50,6 +50,10 @@ namespace gcache
 
         void  reset();
 
+        void  seqno_lock(seqno_t) {}
+
+        void  seqno_unlock() {}
+
         void  set_page_size (size_t size) { page_size_ = size; }
 
         void  set_keep_size (size_t size) { keep_size_ = size; }
