@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2021 Codership Oy <info@codership.com>
+ * Copyright (C) 2010-2024 Codership Oy <info@codership.com>
  */
 
 #include "gcache_rb_store.hpp"
@@ -70,6 +70,7 @@ namespace gcache
         next_      (first_),
         seqno2ptr_ (seqno2ptr),
         gid_       (gid),
+        seqno_locked_(SEQNO_MAX),
         size_cache_(end_ - start_ - sizeof(BufferHeader)),
         size_free_ (size_cache_),
         size_used_ (0),
@@ -118,7 +119,7 @@ namespace gcache
 
             BufferHeader* const bh(ptr2BH(*j));
 
-            if (gu_likely (BH_is_released(bh)))
+            if (gu_likely (BH_is_released(bh) && bh->seqno_g < seqno_locked_))
             {
                 seqno2ptr_.erase (j);
 
