@@ -32,11 +32,14 @@
 
 using namespace gcs::core;
 
-bool
-gcs_core_register (gu_config_t* conf)
+void
+gcs_core_register(gu::Config& conf)
 {
-    gcs_group_register(reinterpret_cast<gu::Config*>(conf));
-    return (gcs_backend_register(conf));
+    gcs_group::register_params(conf);
+    if (gcs_backend_register(reinterpret_cast<gu_config_t*>(&conf)))
+    {
+        gu_throw_fatal << "Could not register backend parmeters";
+    }
 }
 
 const size_t CORE_FIFO_LEN = (1 << 10); // 1024 elements (no need to have more)
@@ -54,9 +57,21 @@ core_state_t;
 
 struct gcs_core
 {
+    gcs_core(gu::Config&  conf,
+             gcache_t*    cache,
+             const char*  node_name,
+             const char*  inc_addr,
+             int          repl_proto_ver,
+             int          appl_proto_ver,
+             int          gcs_proto_ver = GCS_PROTO_MAX);
+    ~gcs_core();
+
     gu_config_t*    config;
     gcache_t*       cache;
 
+    /* group context */
+    gcs_group_t     group;
+
     /* connection per se */
     long            prim_comp_no;
     core_state_t    state;
@@ -81,9 +96,6 @@ struct gcs_core
     /* local action FIFO */
     gcs_fifo_lite_t* fifo;
 
-    /* group context */
-    gcs_group_t     group;
-
     /* backend part */
     size_t          msg_size;
     gcs_backend_t   backend;   // message IO context
@@ -112,24 +124,37 @@ typedef struct causal_act
     gu_cond_t*   cond;
 } causal_act_t;
 
-gcs_core_t*
-gcs_core_create (gu_config_t* const conf,
-                 gcache_t*    const cache,
-                 const char*  const node_name,
-                 const char*  const inc_addr,
-                 int          const repl_proto_ver,
-                 int          const appl_proto_ver,
-                 int          const gcs_proto_ver)
+gcs_core::gcs_core(gu::Config&  conf,
+                   gcache_t*    cache,
+                   const char*  node_name,
+                   const char*  inc_addr,
+                   int          repl_proto_ver,
+                   int          appl_proto_ver,
+                   int          gcs_proto_ver)
+    :
+    config(reinterpret_cast<gu_config_t*>(&conf)),
+    cache(cache),
+    group(conf, cache, node_name, inc_addr,
+          gcs_proto_ver, repl_proto_ver,appl_proto_ver),
+    prim_comp_no(),
+    state(),
+    proto_ver(),
+    send_lock(),
+    send_buf(),
+    send_buf_len(),
+    send_act_no(),
+    recv_msg(),
+    code_msg_buf(),
+    fifo(),
+    msg_size(),
+    backend()
+#ifdef GCS_CORE_TESTING
+    ,ls()        // to lock-step in unit tests
+    ,state_uuid()
+#endif
 {
-    assert (conf);
-
-    gcs_core_t* core = GU_CALLOC (1, gcs_core_t);
-
-    if (NULL != core) {
-
-        core->config = conf;
-        core->cache  = cache;
-
+    auto core(this); // to minimize diff
+    {
         // Need to allocate something, otherwise Spread 3.17.3 freaks out.
         core->recv_msg.buf = gu_malloc(CORE_INIT_BUF_SIZE);
         if (core->recv_msg.buf) {
@@ -147,20 +172,13 @@ gcs_core_create (gu_config_t* const conf,
                     gu_mutex_init  (&core->send_lock, NULL);
                     core->proto_ver = -1;
                     // ^^^ shall be bumped in gcs_group_act_conf()
-
-                    gcs_group_init (&core->group,
-                                    reinterpret_cast<gu::Config*>(conf), cache,
-                                    node_name, inc_addr,
-                                    gcs_proto_ver, repl_proto_ver,appl_proto_ver
-                        );
-
                     core->state = CORE_CLOSED;
                     core->send_act_no = 1; // 0 == no actions sent
 #ifdef GCS_CORE_TESTING
                     gu_lock_step_init (&core->ls);
                     core->state_uuid = GU_UUID_NIL;
 #endif
-                    return core; // success
+                    return; // success
                 }
 
                 gu_free (core->send_buf);
@@ -168,11 +186,27 @@ gcs_core_create (gu_config_t* const conf,
 
             gu_free (core->recv_msg.buf);
         }
-
-        gu_free (core);
     }
 
-    return NULL; // failure
+    gu_throw_fatal << "Failed to initialize GCS core";
+}
+
+gcs_core_t*
+gcs_core_create (gu::Config&  conf,
+                 gcache_t*    const cache,
+                 const char*  const node_name,
+                 const char*  const inc_addr,
+                 int          const repl_proto_ver,
+                 int          const appl_proto_ver,
+                 int          const gcs_proto_ver)
+{
+    try {
+        return new gcs_core(conf, cache, node_name, inc_addr,
+                            repl_proto_ver, appl_proto_ver, gcs_proto_ver);
+    }
+    catch (...) {
+        return nullptr;
+    }
 }
 
 long
@@ -1007,7 +1041,9 @@ core_handle_state_msg (gcs_core_t*          core,
     if (GCS_GROUP_WAIT_STATE_MSG == gcs_group_state (group))
     {
         if (gu_mutex_lock (&core->send_lock)) abort();
-        ret = gcs_group_handle_state_msg (group, msg);
+        // cast to int is needed to distinguish between positive enums and
+        // negative error codes
+        ret = int(gcs_group_handle_state_msg (group, msg));
 
         switch (ret) {
         case GCS_GROUP_PRIMARY:
@@ -1306,6 +1342,8 @@ out:
             conn->backend.close(&conn->backend);
             if (GCS_ACT_INCONSISTENCY != recv_act->act.type) {
                 /* inconsistency event must be passed up */
+                gu_fatal("Unrecoverable error happened above. Aborting...");
+                usleep(1000000); // give it a second
                 gu_abort();
             }
         }
@@ -1333,12 +1371,11 @@ long gcs_core_close (gcs_core_t* core)
     return ret;
 }
 
-long gcs_core_destroy (gcs_core_t* core)
+static int
+core_destroy(gcs_core_t* core)
 {
     core_act_t* tmp;
 
-    if (!core) return -EBADFD;
-
     if (gu_mutex_lock (&core->send_lock)) return -EBADFD;
     {
         if (CORE_CLOSED != core->state) {
@@ -1366,7 +1403,6 @@ long gcs_core_destroy (gcs_core_t* core)
         gcs_fifo_lite_pop_head (core->fifo);
     }
     gcs_fifo_lite_destroy (core->fifo);
-    gcs_group_free (&core->group);
 
     /* free buffers */
     gu_free (core->recv_msg.buf);
@@ -1376,11 +1412,28 @@ long gcs_core_destroy (gcs_core_t* core)
     gu_lock_step_destroy (&core->ls);
 #endif
 
-    gu_free (core);
-
     return 0;
 }
 
+gcs_core::~gcs_core()
+{
+    int const ret(core_destroy(this));
+    if (ret) {
+        gu_throw_error(ret) << "GCS core destructor failed";
+    }
+}
+
+long gcs_core_destroy (gcs_core_t* core)
+{
+    try {
+        delete core;
+        return 0;
+    }
+    catch (...) {
+        return -1;
+    }
+}
+
 int
 gcs_core_proto_ver (const gcs_core_t* conn)
 {
@@ -1611,6 +1664,11 @@ void gcs_core_get_status(gcs_core_t* core, gu::Status& status)
     gu_mutex_unlock(&core->send_lock);
 }
 
+void gcs_core_get_protocols(gcs_core_t* core, int& appl, int& repl, int& gcs)
+{
+    core->group.get_protocols(appl, repl, gcs);
+}
+
 #ifdef GCS_CORE_TESTING
 
 gcs_backend_t*
