@@ -127,7 +127,7 @@ gcs_sm_open (gcs_sm_t* sm)
 
     gu_mutex_unlock (&sm->lock);
 
-    if (ret) { gu_error ("Can't open send monitor: wrong state %d", ret); }
+    if (ret) { gu_error ("Can't open send monitor: wrong state %ld", ret); }
 
     return ret;
 }
